@@ -33,6 +33,90 @@ def test_default_versions(isolation):
     assert builder.get_default_versions() == ['standard']
 
 
+class TestDefaultFileSelection:
+    def test_flat_layout(self, temp_dir):
+        config = {'project': {'name': 'my-app', 'version': '0.0.1'}}
+        builder = WheelBuilder(str(temp_dir), config=config)
+
+        flat_root = temp_dir / 'my_app' / '__init__.py'
+        flat_root.ensure_parent_dir_exists()
+        flat_root.touch()
+
+        src_root = temp_dir / 'src' / 'my_app' / '__init__.py'
+        src_root.ensure_parent_dir_exists()
+        src_root.touch()
+
+        single_module_root = temp_dir / 'my_app.py'
+        single_module_root.touch()
+
+        namespace_root = temp_dir / 'ns' / 'my_app' / '__init__.py'
+        namespace_root.ensure_parent_dir_exists()
+        namespace_root.touch()
+
+        assert builder.config.default_include() == []
+        assert builder.config.default_exclude() == []
+        assert builder.config.default_packages() == ['my_app']
+
+    def test_src_layout(self, temp_dir):
+        config = {'project': {'name': 'my-app', 'version': '0.0.1'}}
+        builder = WheelBuilder(str(temp_dir), config=config)
+
+        src_root = temp_dir / 'src' / 'my_app' / '__init__.py'
+        src_root.ensure_parent_dir_exists()
+        src_root.touch()
+
+        single_module_root = temp_dir / 'my_app.py'
+        single_module_root.touch()
+
+        namespace_root = temp_dir / 'ns' / 'my_app' / '__init__.py'
+        namespace_root.ensure_parent_dir_exists()
+        namespace_root.touch()
+
+        assert builder.config.default_include() == []
+        assert builder.config.default_exclude() == []
+        assert builder.config.default_packages() == ['src/my_app']
+
+    def test_single_module(self, temp_dir):
+        config = {'project': {'name': 'my-app', 'version': '0.0.1'}}
+        builder = WheelBuilder(str(temp_dir), config=config)
+
+        single_module_root = temp_dir / 'my_app.py'
+        single_module_root.touch()
+
+        namespace_root = temp_dir / 'ns' / 'my_app' / '__init__.py'
+        namespace_root.ensure_parent_dir_exists()
+        namespace_root.touch()
+
+        assert builder.config.default_include() == []
+        assert builder.config.default_exclude() == []
+        assert builder.config.default_packages() == ['my_app.py']
+
+    def test_namespace(self, temp_dir):
+        config = {'project': {'name': 'my-app', 'version': '0.0.1'}}
+        builder = WheelBuilder(str(temp_dir), config=config)
+
+        namespace_root = temp_dir / 'ns' / 'my_app' / '__init__.py'
+        namespace_root.ensure_parent_dir_exists()
+        namespace_root.touch()
+
+        assert builder.config.default_include() == []
+        assert builder.config.default_exclude() == []
+        assert builder.config.default_packages() == ['ns']
+
+    def test_default(self, temp_dir):
+        config = {'project': {'name': 'my-app', 'version': '0.0.1'}}
+        builder = WheelBuilder(str(temp_dir), config=config)
+
+        for i in range(2):
+            namespace_root = temp_dir / f'ns{i}' / 'my_app' / '__init__.py'
+            namespace_root.ensure_parent_dir_exists()
+            namespace_root.touch()
+
+        assert builder.config.default_include() == ['*.py']
+        assert builder.config.default_exclude() == ['test*']
+        assert builder.config.default_packages() == []
+
+
 class TestCoreMetadataConstructor:
     def test_default(self, isolation):
         builder = WheelBuilder(str(isolation))
@@ -147,7 +231,7 @@ class TestSharedData:
 
         assert builder.config.shared_data == {
             str(isolation / 'source2'): 'target1',
-            str(isolation / 'very-nested'): f'target1{os.path.sep}embedded',
+            str(isolation / 'very-nested'): f'target1{os.sep}embedded',
             str(isolation / 'source1'): 'target2',
         }
 
@@ -236,7 +320,7 @@ class TestExtraMetadata:
 
         assert builder.config.extra_metadata == {
             str(isolation / 'source2'): 'target1',
-            str(isolation / 'very-nested'): f'target1{os.path.sep}embedded',
+            str(isolation / 'very-nested'): f'target1{os.sep}embedded',
             str(isolation / 'source1'): 'target2',
         }
 
@@ -1205,6 +1289,7 @@ class TestBuildStandard:
                     def initialize(self, version, build_data):
                         build_data['pure_python'] = False
                         build_data['infer_tag'] = True
+                        build_data['artifacts'].extend(('lib.so', 'lib.h'))
                         build_data['force_include']['../artifacts'] = 'my_app'
 
                         artifact_path = pathlib.Path('..', 'artifacts')
@@ -1289,11 +1374,12 @@ class TestBuildStandard:
                     def initialize(self, version, build_data):
                         build_data['pure_python'] = False
                         build_data['infer_tag'] = True
-                        build_data['artifacts'] = ['src/my_app/lib.so', 'src/lib.pyd']
+                        build_data['artifacts'] = ['src/my_app/lib.so']
+                        build_data['force_include']['src/zlib.pyd'] = 'src/zlib.pyd'
 
                         pathlib.Path('src', 'my_app', 'lib.so').touch()
                         pathlib.Path('src', 'lib.h').touch()
-                        pathlib.Path('src', 'lib.pyd').touch()
+                        pathlib.Path('src', 'zlib.pyd').touch()
                 """
             )
         )
@@ -1750,6 +1836,73 @@ class TestBuildStandard:
             zip_info = zip_archive.getinfo(f'{metadata_directory}/WHEEL')
             assert zip_info.date_time == (2020, 2, 2, 0, 0, 0)
 
+    @fixed_pathlib_resolution
+    def test_editable_default_force_include_option(self, hatch, helpers, temp_dir, config_file):
+        config_file.model.template.plugins['default']['src-layout'] = True
+        config_file.save()
+
+        project_name = 'My App'
+
+        with temp_dir.as_cwd():
+            result = hatch('new', project_name)
+
+        assert result.exit_code == 0, result.output
+
+        project_path = temp_dir / 'my-app'
+
+        config = {
+            'project': {'name': 'my__app', 'dynamic': ['version']},
+            'tool': {
+                'hatch': {
+                    'version': {'path': 'src/my_app/__about__.py'},
+                    'build': {
+                        'targets': {
+                            'wheel': {
+                                'versions': ['editable'],
+                                'force-include': {'src/my_app/__about__.py': 'zfoo.py'},
+                            }
+                        }
+                    },
+                },
+            },
+        }
+        builder = WheelBuilder(str(project_path), config=config)
+
+        build_path = project_path / 'dist'
+        build_path.mkdir()
+
+        with project_path.as_cwd():
+            artifacts = list(builder.build(str(build_path)))
+
+        assert len(artifacts) == 1
+        expected_artifact = artifacts[0]
+
+        build_artifacts = list(build_path.iterdir())
+        assert len(build_artifacts) == 1
+        assert expected_artifact == str(build_artifacts[0])
+        assert expected_artifact == str(build_path / f'{builder.project_id}-{get_python_versions_tag()}-none-any.whl')
+
+        extraction_directory = temp_dir / '_archive'
+        extraction_directory.mkdir()
+
+        with zipfile.ZipFile(str(expected_artifact), 'r') as zip_archive:
+            zip_archive.extractall(str(extraction_directory))
+
+        metadata_directory = f'{builder.project_id}.dist-info'
+        expected_files = helpers.get_template_files(
+            'wheel.standard_editable_pth_force_include',
+            project_name,
+            metadata_directory=metadata_directory,
+            package_paths=[str(project_path / 'src')],
+        )
+        helpers.assert_files(extraction_directory, expected_files, check_contents=True)
+
+        # Inspect the archive rather than the extracted files because on Windows they lose their metadata
+        # https://stackoverflow.com/q/9813243
+        with zipfile.ZipFile(str(expected_artifact), 'r') as zip_archive:
+            zip_info = zip_archive.getinfo(f'{metadata_directory}/WHEEL')
+            assert zip_info.date_time == (2020, 2, 2, 0, 0, 0)
+
     @fixed_pathlib_resolution
     def test_editable_exact(self, hatch, helpers, temp_dir):
         project_name = 'My App'
@@ -1960,6 +2113,153 @@ class TestBuildStandard:
             zip_info = zip_archive.getinfo(f'{metadata_directory}/WHEEL')
             assert zip_info.date_time == (2020, 2, 2, 0, 0, 0)
 
+    @fixed_pathlib_resolution
+    def test_editable_exact_force_include_option(self, hatch, helpers, temp_dir):
+        project_name = 'My App'
+
+        with temp_dir.as_cwd():
+            result = hatch('new', project_name)
+
+        assert result.exit_code == 0, result.output
+
+        project_path = temp_dir / 'my-app'
+
+        config = {
+            'project': {'name': 'my__app', 'dynamic': ['version']},
+            'tool': {
+                'hatch': {
+                    'version': {'path': 'my_app/__about__.py'},
+                    'build': {
+                        'targets': {
+                            'wheel': {
+                                'versions': ['editable'],
+                                'dev-mode-exact': True,
+                                'force-include': {'my_app/__about__.py': 'zfoo.py'},
+                            }
+                        }
+                    },
+                },
+            },
+        }
+        builder = WheelBuilder(str(project_path), config=config)
+
+        build_path = project_path / 'dist'
+        build_path.mkdir()
+
+        with project_path.as_cwd():
+            artifacts = list(builder.build(str(build_path)))
+
+        assert len(artifacts) == 1
+        expected_artifact = artifacts[0]
+
+        build_artifacts = list(build_path.iterdir())
+        assert len(build_artifacts) == 1
+        assert expected_artifact == str(build_artifacts[0])
+        assert expected_artifact == str(build_path / f'{builder.project_id}-{get_python_versions_tag()}-none-any.whl')
+
+        extraction_directory = temp_dir / '_archive'
+        extraction_directory.mkdir()
+
+        with zipfile.ZipFile(str(expected_artifact), 'r') as zip_archive:
+            zip_archive.extractall(str(extraction_directory))
+
+        metadata_directory = f'{builder.project_id}.dist-info'
+        expected_files = helpers.get_template_files(
+            'wheel.standard_editable_exact_force_include',
+            project_name,
+            metadata_directory=metadata_directory,
+            package_root=str(project_path / 'my_app' / '__init__.py'),
+        )
+        helpers.assert_files(extraction_directory, expected_files, check_contents=True)
+
+        # Inspect the archive rather than the extracted files because on Windows they lose their metadata
+        # https://stackoverflow.com/q/9813243
+        with zipfile.ZipFile(str(expected_artifact), 'r') as zip_archive:
+            zip_info = zip_archive.getinfo(f'{metadata_directory}/WHEEL')
+            assert zip_info.date_time == (2020, 2, 2, 0, 0, 0)
+
+    @fixed_pathlib_resolution
+    def test_editable_exact_force_include_build_data_precedence(self, hatch, helpers, temp_dir):
+        project_name = 'My App'
+
+        with temp_dir.as_cwd():
+            result = hatch('new', project_name)
+
+        assert result.exit_code == 0, result.output
+
+        project_path = temp_dir / 'my-app'
+
+        build_script = project_path / DEFAULT_BUILD_SCRIPT
+        build_script.write_text(
+            helpers.dedent(
+                """
+                import pathlib
+
+                from hatchling.builders.hooks.plugin.interface import BuildHookInterface
+
+                class CustomHook(BuildHookInterface):
+                    def initialize(self, version, build_data):
+                        # Prefix z just to satisfy our ordering test assertion
+                        build_data['force_include_editable']['my_app/__about__.py'] = 'zfoo.py'
+                """
+            )
+        )
+
+        config = {
+            'project': {'name': 'my__app', 'dynamic': ['version']},
+            'tool': {
+                'hatch': {
+                    'version': {'path': 'my_app/__about__.py'},
+                    'build': {
+                        'targets': {
+                            'wheel': {
+                                'versions': ['editable'],
+                                'dev-mode-exact': True,
+                                'force-include': {'my_app/__about__.py': 'zbar.py'},
+                                'hooks': {'custom': {}},
+                            }
+                        }
+                    },
+                },
+            },
+        }
+        builder = WheelBuilder(str(project_path), config=config)
+
+        build_path = project_path / 'dist'
+        build_path.mkdir()
+
+        with project_path.as_cwd():
+            artifacts = list(builder.build(str(build_path)))
+
+        assert len(artifacts) == 1
+        expected_artifact = artifacts[0]
+
+        build_artifacts = list(build_path.iterdir())
+        assert len(build_artifacts) == 1
+        assert expected_artifact == str(build_artifacts[0])
+        assert expected_artifact == str(build_path / f'{builder.project_id}-{get_python_versions_tag()}-none-any.whl')
+
+        extraction_directory = temp_dir / '_archive'
+        extraction_directory.mkdir()
+
+        with zipfile.ZipFile(str(expected_artifact), 'r') as zip_archive:
+            zip_archive.extractall(str(extraction_directory))
+
+        metadata_directory = f'{builder.project_id}.dist-info'
+        expected_files = helpers.get_template_files(
+            'wheel.standard_editable_exact_force_include',
+            project_name,
+            metadata_directory=metadata_directory,
+            package_root=str(project_path / 'my_app' / '__init__.py'),
+        )
+        helpers.assert_files(extraction_directory, expected_files, check_contents=True)
+
+        # Inspect the archive rather than the extracted files because on Windows they lose their metadata
+        # https://stackoverflow.com/q/9813243
+        with zipfile.ZipFile(str(expected_artifact), 'r') as zip_archive:
+            zip_info = zip_archive.getinfo(f'{metadata_directory}/WHEEL')
+            assert zip_info.date_time == (2020, 2, 2, 0, 0, 0)
+
     @fixed_pathlib_resolution
     def test_editable_pth(self, hatch, helpers, temp_dir, config_file):
         config_file.model.template.plugins['default']['src-layout'] = True
@@ -2072,3 +2372,151 @@ class TestBuildStandard:
             namespace='namespace',
         )
         helpers.assert_files(extraction_directory, expected_files, check_contents=True)
+
+    def test_default_entry_points(self, hatch, helpers, temp_dir):
+        project_name = 'My App'
+
+        with temp_dir.as_cwd():
+            result = hatch('new', project_name)
+
+        assert result.exit_code == 0, result.output
+
+        project_path = temp_dir / 'my-app'
+
+        config = {
+            'project': {'name': 'my__app', 'dynamic': ['version'], 'scripts': {'foo': 'pkg:bar', 'bar': 'pkg:foo'}},
+            'tool': {
+                'hatch': {
+                    'version': {'path': 'my_app/__about__.py'},
+                    'build': {'targets': {'wheel': {'versions': ['standard']}}},
+                },
+            },
+        }
+        builder = WheelBuilder(str(project_path), config=config)
+
+        build_path = project_path / 'dist'
+
+        with project_path.as_cwd():
+            artifacts = list(builder.build())
+
+        assert len(artifacts) == 1
+        expected_artifact = artifacts[0]
+
+        build_artifacts = list(build_path.iterdir())
+        assert len(build_artifacts) == 1
+        assert expected_artifact == str(build_artifacts[0])
+        assert expected_artifact == str(build_path / f'{builder.project_id}-{get_python_versions_tag()}-none-any.whl')
+
+        extraction_directory = temp_dir / '_archive'
+        extraction_directory.mkdir()
+
+        with zipfile.ZipFile(str(expected_artifact), 'r') as zip_archive:
+            zip_archive.extractall(str(extraction_directory))
+
+        metadata_directory = f'{builder.project_id}.dist-info'
+        expected_files = helpers.get_template_files(
+            'wheel.standard_entry_points', project_name, metadata_directory=metadata_directory
+        )
+        helpers.assert_files(extraction_directory, expected_files, check_contents=True)
+
+    def test_explicit_selection_with_src_layout(self, hatch, helpers, temp_dir, config_file):
+        config_file.model.template.plugins['default']['src-layout'] = True
+        config_file.save()
+
+        project_name = 'My App'
+
+        with temp_dir.as_cwd():
+            result = hatch('new', project_name)
+
+        assert result.exit_code == 0, result.output
+
+        project_path = temp_dir / 'my-app'
+
+        config = {
+            'project': {'name': 'my__app', 'dynamic': ['version']},
+            'tool': {
+                'hatch': {
+                    'version': {'path': 'src/my_app/__about__.py'},
+                    'build': {
+                        'targets': {
+                            'wheel': {
+                                'versions': ['standard'],
+                                'artifacts': ['README.md'],
+                                'only-include': ['src/my_app'],
+                                'sources': ['src'],
+                            }
+                        },
+                    },
+                },
+            },
+        }
+        builder = WheelBuilder(str(project_path), config=config)
+
+        build_path = project_path / 'dist'
+        build_path.mkdir()
+
+        with project_path.as_cwd():
+            artifacts = list(builder.build(str(build_path)))
+
+        assert len(artifacts) == 1
+        expected_artifact = artifacts[0]
+
+        build_artifacts = list(build_path.iterdir())
+        assert len(build_artifacts) == 1
+        assert expected_artifact == str(build_artifacts[0])
+
+        extraction_directory = temp_dir / '_archive'
+        extraction_directory.mkdir()
+
+        with zipfile.ZipFile(str(expected_artifact), 'r') as zip_archive:
+            zip_archive.extractall(str(extraction_directory))
+
+        metadata_directory = f'{builder.project_id}.dist-info'
+        expected_files = helpers.get_template_files(
+            'wheel.standard_default_license_single',
+            project_name,
+            metadata_directory=metadata_directory,
+        )
+        helpers.assert_files(extraction_directory, expected_files, check_contents=True)
+
+    def test_single_module(self, hatch, helpers, temp_dir, config_file):
+        project_name = 'My App'
+
+        with temp_dir.as_cwd():
+            result = hatch('new', project_name)
+
+        assert result.exit_code == 0, result.output
+
+        project_path = temp_dir / 'my-app'
+        (project_path / 'my_app').remove()
+        (project_path / 'my_app.py').touch()
+
+        config = {'project': {'name': 'my__app', 'version': '0.0.1'}}
+        builder = WheelBuilder(str(project_path), config=config)
+
+        build_path = project_path / 'dist'
+        build_path.mkdir()
+
+        with project_path.as_cwd():
+            artifacts = list(builder.build(str(build_path)))
+
+        assert len(artifacts) == 1
+        expected_artifact = artifacts[0]
+
+        build_artifacts = list(build_path.iterdir())
+        assert len(build_artifacts) == 1
+        assert expected_artifact == str(build_artifacts[0])
+
+        extraction_directory = temp_dir / '_archive'
+        extraction_directory.mkdir()
+
+        with zipfile.ZipFile(str(expected_artifact), 'r') as zip_archive:
+            zip_archive.extractall(str(extraction_directory))
+
+        metadata_directory = f'{builder.project_id}.dist-info'
+        expected_files = helpers.get_template_files(
+            'wheel.standard_default_single_module',
+            project_name,
+            metadata_directory=metadata_directory,
+        )
+        helpers.assert_files(extraction_directory, expected_files, check_contents=True)
