@@ -15,9 +15,18 @@ PUBLISHER_TOKEN = os.environ.get('HATCH_CI_PUBLISHER_TOKEN')
 
 pytestmark = [
     pytest.mark.skipif(not PUBLISHER_TOKEN, reason='Publishing tests are only executed within CI environments'),
+    pytest.mark.xfail(reason='Test PyPI can be flaky'),
 ]
 
 
+@pytest.fixture(autouse=True)
+def local_builder(mock_backend_process, mocker):
+    if mock_backend_process:
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.build_environment')
+
+    yield
+
+
 @pytest.fixture(autouse=True)
 def keyring_store(mocker):
     mock_store = defaultdict(dict)
@@ -101,7 +110,7 @@ def test_explicit_options(hatch, temp_dir):
 
     assert result.exit_code == 1, result.output
     assert result.output == (
-        'Use the standard CLI flags rather than passing explicit options when using the `pypi` plugin\n'
+        'Use the standard CLI flags rather than passing explicit options when using the `index` plugin\n'
     )
 
 
@@ -121,6 +130,25 @@ def test_unknown_publisher(hatch, temp_dir):
     assert result.output == 'Unknown publisher: foo\n'
 
 
+def test_disabled(hatch, temp_dir, config_file):
+    config_file.model.publish['index']['disable'] = True
+    config_file.save()
+
+    project_name = 'My App'
+
+    with temp_dir.as_cwd():
+        result = hatch('new', project_name)
+        assert result.exit_code == 0, result.output
+
+    path = temp_dir / 'my-app'
+
+    with path.as_cwd():
+        result = hatch('publish', '-n')
+
+    assert result.exit_code == 1, result.output
+    assert result.output == 'Publisher is disabled: index\n'
+
+
 def test_missing_user(hatch, temp_dir):
     project_name = 'My App'
 
@@ -188,9 +216,9 @@ def test_flags(hatch, temp_dir_cache, helpers, published_project_name):
 
 
 def test_plugin_config(hatch, temp_dir_cache, helpers, published_project_name, config_file):
-    config_file.model.publish['pypi']['user'] = '__token__'
-    config_file.model.publish['pypi']['auth'] = PUBLISHER_TOKEN
-    config_file.model.publish['pypi']['repo'] = 'test'
+    config_file.model.publish['index']['user'] = '__token__'
+    config_file.model.publish['index']['auth'] = PUBLISHER_TOKEN
+    config_file.model.publish['index']['repo'] = 'test'
     config_file.save()
 
     with temp_dir_cache.as_cwd():
@@ -397,6 +425,87 @@ def test_no_artifacts(hatch, temp_dir_cache, helpers, published_project_name):
     )
 
 
+def test_enable_with_flag(hatch, temp_dir_cache, helpers, published_project_name, config_file):
+    config_file.model.publish['index']['user'] = '__token__'
+    config_file.model.publish['index']['auth'] = PUBLISHER_TOKEN
+    config_file.model.publish['index']['repo'] = 'test'
+    config_file.model.publish['index']['disable'] = True
+    config_file.save()
+
+    with temp_dir_cache.as_cwd():
+        result = hatch('new', published_project_name)
+        assert result.exit_code == 0, result.output
+
+    path = temp_dir_cache / published_project_name
+
+    with path.as_cwd():
+        del os.environ[PublishEnvVars.REPO]
+
+        current_version = timestamp_to_version(helpers.get_current_timestamp())
+        result = hatch('version', current_version)
+        assert result.exit_code == 0, result.output
+
+        result = hatch('build')
+        assert result.exit_code == 0, result.output
+
+        build_directory = path / 'dist'
+        artifacts = list(build_directory.iterdir())
+
+        result = hatch('publish', '-y')
+
+    assert result.exit_code == 0, result.output
+    assert result.output == helpers.dedent(
+        f"""
+        {artifacts[0].relative_to(path)} ... success
+        {artifacts[1].relative_to(path)} ... success
+
+        [{published_project_name}]
+        https://test.pypi.org/project/{published_project_name}/{current_version}/
+        """
+    )
+
+
+def test_enable_with_prompt(hatch, temp_dir_cache, helpers, published_project_name, config_file):
+    config_file.model.publish['index']['user'] = '__token__'
+    config_file.model.publish['index']['auth'] = PUBLISHER_TOKEN
+    config_file.model.publish['index']['repo'] = 'test'
+    config_file.model.publish['index']['disable'] = True
+    config_file.save()
+
+    with temp_dir_cache.as_cwd():
+        result = hatch('new', published_project_name)
+        assert result.exit_code == 0, result.output
+
+    path = temp_dir_cache / published_project_name
+
+    with path.as_cwd():
+        del os.environ[PublishEnvVars.REPO]
+
+        current_version = timestamp_to_version(helpers.get_current_timestamp())
+        result = hatch('version', current_version)
+        assert result.exit_code == 0, result.output
+
+        result = hatch('build')
+        assert result.exit_code == 0, result.output
+
+        build_directory = path / 'dist'
+        artifacts = list(build_directory.iterdir())
+
+        result = hatch('publish', input='y\n')
+
+    assert result.exit_code == 0, result.output
+    assert result.output == helpers.dedent(
+        f"""
+        Confirm `index` publishing [y/N]: y
+        {artifacts[0].relative_to(path)} ... success
+        {artifacts[1].relative_to(path)} ... success
+
+        [{published_project_name}]
+        https://test.pypi.org/project/{published_project_name}/{current_version}/
+        """
+    )
+
+
 class TestWheel:
     @pytest.mark.parametrize('field', ['name', 'version'])
     def test_missing_required_metadata_field(self, hatch, temp_dir_cache, helpers, published_project_name, field):
@@ -469,7 +578,7 @@ class TestSourceDistribution:
         metadata_file_path.write_text(remove_metadata_field(field, metadata_file_path.read_text()))
 
         with tarfile.open(artifact_path, 'w:gz') as tar_archive:
-            tar_archive.add(extraction_directory)
+            tar_archive.add(extraction_directory, arcname='')
 
         with path.as_cwd():
             result = hatch('publish', '--user', '__token__', '--auth', 'foo')
