@@ -22,13 +22,12 @@ def get_files(**kwargs):
 
     files.append(File(Path(kwargs['package_name'], 'lib.so'), ''))
     files.append(File(Path(kwargs['package_name'], 'lib.h'), ''))
-    files.append(File(Path(metadata_directory, 'entry_points.txt'), ''))
     files.append(
         File(
             Path(metadata_directory, 'WHEEL'),
             f"""\
 Wheel-Version: 1.0
-Generator: hatch {__version__}
+Generator: hatchling {__version__}
 Root-Is-Purelib: false
 Tag: {kwargs.get('tag', '')}
 """,
